/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.transitions.IBasicTween;
import com.bourre.transitions.IFrameBeacon;
import com.bourre.transitions.IFrameListener;
import com.bourre.core.ISetter;
import com.bourre.core.SetterFactory;
import com.bourre.core.PropertySetter;
import com.bourre.log.Logger;
import com.bourre.log.LogLevel;

class com.bourre.transitions.AbstractTween
	implements IBasicTween, IFrameListener
{	
	private var _nS:Number, _nE:Number, _nRS:Number, _nRE:Number;
	private var _nRate:Number;
	private var _fE:Function;
	private var _bP:Boolean;
	
	private var _oSetter:ISetter;
	
	// virtual members
	private var _isMotionFinished:Function;
	private var _onUpdate:Function;
	private var _oBeacon:IFrameBeacon;
	
	/*
	 * Make tween on object property value.
	 * <strong>Usage :</strong> <em>var t:BasicTweenFPS = new BasicTweenFPS(mc, '_alpha', 0, 30, 100);</em>
	 * @param oT 	Object which the Tween targets.
	 * @param sP 	Setter (method or property).
	 * @param nE 	Ending value of property.
	 * @param nRate Length of time of the motion in frames.
	 * @param nS 	Starting value of property (optional).
	 * @param fE 	Easing function (optional).
	*/
	private function AbstractTween(oT, sP, nE:Number, nRate:Number, nS:Number, fE:Function)
	{
		_oSetter = SetterFactory.getSetter(oT, sP);
		
		_nRE = nE;
		_nRS = (nS!=undefined)? nS : oT[sP];
		_nRate = nRate;
		setEasing(fE);
		_bP = false;
		
		if (isNaN(_nRS)) Logger.LOG( "**Error** " + toString() + " has no start value.", LogLevel.ERROR);
	}
	
	public static function noEasing(t:Number, b:Number, c:Number, d:Number) : Number 
	{
		return c*t/d + b;
	}
	
	public function onEnterFrame() : Void
	{
		if (_isMotionFinished() )
		{
			_onMotionEnd();
		} else
		{
			_onUpdate();
		}
	}

	private function _onMotionEnd() : Void
	{
		_oBeacon.removeFrameListener(this);
		_oSetter.setValue( _nE );
	}
	
	/**
	 * Set easing function.<br><br>
	 * <strong>Usage :</strong> <em>var t.setEasing(mx.transitions.easing.Elastic.easeOut);</em>
	 * @param f Easing function.
	 */
	public function setEasing(f:Function) : Void
	{
		_fE = (f != undefined) ?  f : AbstractTween.noEasing;
	}
	
	/**
	 * Call start function.<br><br>
	 * <strong>Usage :</strong> <em>t.start();</em>
	 */
	public function start() : Void
	{
		execute();
	}
	
	/**
	 * Stop tweening.<br><br>
	 * <strong>Usage :</strong> <em>t.stop();</em>
	 */
	public function stop() : Void
	{
		_oBeacon.removeFrameListener(this);
		_bP = false;
	}
	
	/**
	 * Play tweening from current property's value without reinitializing.<br><br>
	 * <strong>Usage :</strong> <em>t.resume();</em>
	 */
	public function resume() : Void
	{
		_bP = true;
		_oBeacon.addFrameListener(this);
	}
	
	/**
	 * Start tweening. (Command polymorphism).<br><br>
	 * <strong>Usage :</strong> <em>t.execute();</em>
	 * @see com.bourre.commands.Command
	 */
	public function execute() : Void
	{
		_nS = _nRS;
		_oSetter.setValue( _nS );
		_nE = _nRE;
		_bP = true;
		_oBeacon.addFrameListener(this);
	}
	
	public function getTarget()
	{
		return _oSetter.getTarget();
	}
	
	public function getProperty() : String
	{
		return PropertySetter(_oSetter).getProperty();
	}
	
	public function toString() : String
	{
		return 'AbstractTween' + HashCodeFactory.getKey( this );
	}
}