/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.events.DynBasicEvent;
import com.bourre.events.IEvent;
import com.bourre.events.ListenerArray;
import com.bourre.events.IEventDispatcher;

class com.bourre.events.EventBroadcaster
	implements IEventDispatcher
{
	private var _oL:Object;
	private var _aAll:ListenerArray;
	private var _oE:Object;
	private var _oOwner;
	
	private static var _oI:EventBroadcaster;
	private static var _bInitialization:Function = com.bourre.core.HashCodeFactory;
	
	private static function _buildInstance() : EventBroadcaster
	{
		EventBroadcaster._oI = new EventBroadcaster();
		return _oI;
	}
	
	public static function getInstance() : EventBroadcaster
	{
		return (EventBroadcaster._oI instanceof EventBroadcaster) ? EventBroadcaster._oI : EventBroadcaster._buildInstance();
	}
	
	private function _init() : Void
	{
		_oL = new Object();
		_aAll = new ListenerArray();
		_oE = new Object();
	}
 
 	/**
	 * Dispatch events to suscribed listeners.
	 */
	public function EventBroadcaster( owner )
	{
		_oOwner = owner ? owner : this;
		_init();
	}
	
	/**
	 * Get Array of listeners that suscribed for specifical event or for all events.
	 * @example <code>var a:ListenerArray = e.getListenerArray(MyClass.onSomethingEVENT);</code>
	 * @param t Event type. If event type is undefined, method will return listeners that suscribed for all events.
	 * @return ListenerArray
	 */
	public function getListenerArray(t:String) : ListenerArray
	{
		return (t == undefined) ? _aAll : _oL[t];
	}
	
	/**
	 * Check if listeners Array already exists for specifical event type.
	 * @example <code>var b:Boolean = e.listenerArrayExists(MyClass.onSomethingEVENT);</code>
	 * @param t Event type
	 * @return Boolean
	 */
	public function listenerArrayExists(t:String) : Boolean
	{
		return _oL[t] != undefined;
	}
	
	/**
	 * Add listener for receiving all events.
	 * @example <code>e.addListener(myListener);</code>
	 * @param oL Listener object.
	 */
	public function addListener(oL, f:Function) : Void
	{
		if (f) oL = _getEventProxy.apply( this, arguments );
		if (_aAll.insert(oL)) _clean(f?oL.t.__KEY:oL.__KEY);
	}
	
	/**
	 * Remove listener for receiving all events.
	 * @example <code>e.removeListener(myListener);</code>
	 * @param oL Listener object.
	 */
	public function removeListener(oL) : Void 
	{
		_clean(oL.__KEY);
		_aAll.remove(oL); 
	}
	
	/**
	 * Remove all listeners.
	 * @example <code>e.removeAllListeners();</code>
	 */
	public function removeAllListeners() : Void 
	{
		_init();
	}
   
	/**
	 * Add listener for specifical event.
	 * @example <code>e.addListener(onSomethingEVENT, myListener);</code>
	 * @param t Name of the Event.
	 * @param oL Listener object.
	 */
	public function addEventListener(t:String, oL) : Void 
	{
		var f : Function = arguments[2];
		if (f) oL = _getEventProxy.apply( this, arguments.splice(1) );

		if (!_aAll.listenerExists(oL))
		{
			if (!listenerArrayExists(t)) _oL[t] = new ListenerArray();
			if (getListenerArray(t).insert(oL))
			{
				var n:Number = f ? oL.t.__KEY : oL.__KEY;
				if (_oE[n] == undefined) _oE[n] = new Object();
				_oE[n][t] = oL;
			}
		}
	}

	/**
	 * Remove listener that suscribed for specifical event.
	 * @example <code>e.removeListener(onSomethingEVENT, myListener);</code>
	 * @param t Name of the Event.
	 * @param oL Listener object.
	 */
	public function removeEventListener(t:String, oL) : Void 
	{
		if (listenerArrayExists(t))
		{
			var a:ListenerArray = getListenerArray(t);
			if (a.remove(oL)) 
			{
				delete _oE[oL.__KEY][t];
				if ( a.isEmpty() ) delete _oL[t];
			}
		}
	}
	
	/**
	 * Remove all listeners that suscribed for specifical event.
	 * Usage :</strong> <code>e.removeAllEventListeners(onSomethingEVENT);</code>
	 * @param t Name of the Event.
	 */
	public function removeAllEventListeners(t:String) : Void 
	{
		if (listenerArrayExists(t))
		{
			delete _oL[t];
			for (var n in _oE) delete _oE[n][t];
		}
	}
	
	/**
	 * Broadcast event to suscribed listeners.
	 * @example<code>e.broadcastEvent( new BasicEvent(MyClass.onSomethingEVENT) );</code>
	 * @param e IEvent. Event object must implement IEvent interface.
	 */
	public function broadcastEvent(e:IEvent) : Void
	{
		if (e.getTarget() == undefined) e.setTarget( _oOwner );
		var aL:ListenerArray = getListenerArray(e.getType());
		if (aL != undefined) _broadcast(aL, e);
		if (_aAll.length > 0) _broadcast(_aAll, e);
	}
	
	/**
	 * Wrapper for EventDispatcher polymorphism.
	 * @example<code>e.dispatchEvent( {type:'onSomething', target:this, param:12} );</code>
	 * @param o Event object.
	 */
	public function dispatchEvent(o:Object) : Void 
	{
		var e:DynBasicEvent = new DynBasicEvent(o["type"], o["target"]);
		for (var p:String in o) if (o[p] != "type" && o[p] != "target") e[p] = o[p];
		broadcastEvent( e );
	}
	
	/**
	 * Check if listeners Array are empty.
	 * @example <code>var b:Boolean = e.isEmpty();</code>
	 * @return Boolean
	 */
	public function isEmpty() : Boolean
	{
		var p:String;
		for (p in _oL) if (p) break;
		return _aAll.length == 0 && p == undefined;
	}
	
	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return 'EventBroadcaster' + HashCodeFactory.getKey( this );
	}
	
	//
	private function _broadcast(aL:ListenerArray, e:IEvent) : Void
	{
		var l:Number = aL.length;
		while( --l > -1 )
		{
			var o = aL[l];
			var sType:String = typeof(o);
			if (sType == "object" || sType == "movieclip") 
			{
				if (o.handleEvent != undefined)
				{
					o.handleEvent(e);
				} else
				{
					o[String(e.getType())](e);
				}
			} else
			{
				o.apply( this, [e] );
			}
		}
	}
	
	private function _clean(key:Number) : Void
	{
		if (_oE[key] != undefined) 
		{
			var o:Object = _oE[key];
			for (var p:String in o) removeEventListener( p, o[p] );
			delete _oE[key];
		}
	}
	
	private function _getEventProxy(oL, f:Function) : Function
	{
		return Delegate.create.apply( Delegate, [oL, f].concat(arguments.splice(2)) );
	}
}