/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;

class com.bourre.events.ListenerArray extends Array
{
	/**
	 * Array storing event listeners<br><br>
	 * <strong>Usage :</strong> <em>var a:ListenerArray = new ListenerArray();</em>
	 * @see com.bourre.events.EventBroadcaster
	 */
	public function ListenerArray()
	{
		splice.apply(this, [0, 0].concat(arguments));
	}
	
	/**
	 * Get index number of a listener.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = a.getIndex(oListener);</em>
	 * @param oL Listener to check.
	 * @return Number Returns -1 if the listener isn't referenced in the array.
	 */
	public function getIndex(oL) : Number 
	{
		if (typeof( oL ) == "function") oL = oL.t;

		var l:Number = this.length;
		while ( --l > -1 )
		{
			var o = this[l];
			if (o == oL) 
			{
				return l;
			} else if ( typeof( o ) == "function" )
			{
				if (o.t == oL) return l;
			}
		}
		return -1;
	}
	
	/**
	 * Test if one listener is referenced in the array.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = a.listenerExists(oListener);</em>
	 * @param oL Listener to check.
	 * @return Boolean.
	 */
	public function listenerExists(oL) : Boolean 
	{
		return (getIndex(oL) != -1);
	}
	
	/**
	 * Add listener.<br><br>
	 * <strong>Usage :</strong> <em>a.push(oListener);</em>
	 * @param oL Listener to add.
	 * @return Boolean that indicates if the listener has been successfully added == wasn't already there.
	 */
	public function insert(oL) : Boolean
	{
		if (!listenerExists(oL)) 
		{
			push(oL);
			return true;
		} else return false;
	}

	/**
	 * Remove listener.<br><br>
	 * <strong>Usage :</strong> <em>a.remove(oListener);</em>
	 * @param oL Listener to remove.
	 * @return Boolean that indicates if the listener has been found and was removed.
	 */
	public function remove(oL) : Boolean 
	{
		var i:Number = getIndex(oL);
		if (i != -1) 
		{
			splice(i, 1);
			return true;
		} else return false;
	}
	
	/**
	 * Check if Array's empty.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = a.isEmpty();</em>
	 * @return Boolean.
	 */
	public function isEmpty() : Boolean
	{
		return this.length < 1;
	}
	
	public function toString() : String
	{
		return 'ListenerArray' + HashCodeFactory.getKey( this );
	}
}