/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.log.*;

class com.bourre.structures.Point
{
	public var x : Number;
	public var y : Number;

	/**
	 * Usage : <em>var p:Point = new Point(4, 2);</em>
	 * @param nX 					x property value.
	 * @param nY 					y property value.
	 */
	public function Point (nX:Number, nY:Number) 
	{
		init(nX, nY);
	}


	/**
	 * To init the x and y properties.<br><br>
	 * <strong>Usage :</strong> <em>p.init(5, 6);</em>
	 * @param nX					Number
	 * @param nY					Number
	 */
	public function init (nX:Number, nY:Number) : Void 
	{ 
		x = nX;
		y = nY;
	}

	/**
	 * To init the x and y properties.<br><br>
	 * <strong>Usage :</strong> <em>p.reset( myPoint );</em>
	 * @param v						Point
	 */
	public function reset(p:Point) : Void
	{
		init(p.x, p.y);
	}

	/**
	 * To check equality of properties with another Point object.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = p1.equals(p2);</em>
	 * @param p						Point object for checking equality.
	 * @return Boolean.
	 */
	public function equals(p:Point) : Boolean 
	{ 
		return (x == p.x && y == p.y); 
	}
	
	/**
	 * To check equality beetween properties values (x, y) and two given numbers.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = p1.equals(5, 3);</em>
	 * @param nX					Number (x value)
	 * @param nY					Number (y value)
	 * @return Boolean.
	 */
	public function compare(nX:Number, nY:Number) : Boolean 
	{ 
		return (x == nX && y == nY); 
	}
	
	/**
	 * To make a copy of the object.<br><br>
	 * <strong>Usage :</strong> <em>var p2:Point = p1.clone();</em>
	 * @return Point.
	 */
	public function clone() : Point 
	{ 
		return new Point(x, y); 
	}

	/*
	 * Arithmetical operations
	 */
	
	/**
	 * Minus operation beetween the properties of two Point objects.<br><br>
	 * <strong>Usage :</strong> <em>var p3:Point = p1.minus(p2);</em>
	 * @param p						Point object to make the arithmetical operation with.
	 */
	public function minus(p:Point) : Void 
	{ 
		init(x - p.x, y - p.y); 
	}
	
	/**
	 * Plus operation beetween the properties of two Point objects.<br><br>
	 * <strong>Usage :</strong> <em>var p3:Point = p1.plus(p2);</em>
	 * @param p						Point object to make the arithmetical operation with.
	 */
	public function plus(p:Point) : Void 
	{ 
		init(x + p.x, y + p.y); 
	}
	
	/**
	 * Set to negative all the properties.<br><br>
	 * <strong>Usage :</strong> <em>p.neg();</em>
	 */
	public function neg() : Void 
	{ 
		init(-x, -y); 
	}
	
	/**
	 * Set to absolute all the properties.<br><br>
	 * <strong>Usage :</strong> <em>p.abs();</em>
	 */
	public function abs() : Void 
	{ 
		init(Math.abs(x), Math.abs(y)); 
	}
	
	/**
	 * Get length of the vector.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = p.getLength();</em>
	 * @return Number.
	 */
	public function getLength() : Number
	{
		return Math.sqrt( (x*x) + (y*y) );
	}
	
	/**
	 * Transforms vector in  direction.<br><br>
	 * <strong>Usage :</strong> <em>p.normalize();</em>
	 */
	public function normalize() : Void
	{
		var l:Number = getLength();
		if (l/0)
		{
			x /= l;
			y /= l;
		} else
		{
			Logger.LOG("**Warning** " + toString() + ".normalize() was called on a zero-length vector!", LogLevel.WARN);
		}
	}
	
	/**
	 * Get Vector direction.<br><br>
	 * <strong>Usage :</strong> <em>var d:Point = p0.getDirection();</em>
	 */
	public function getDirection() : Point
	{
		var p:Point = clone();
		p.normalize();
		return p;
	}
	
	/**
	 * Get dot product.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = p0.getDotProduct( p1 );</em>
	 * @param 	Point.
	 * @return 	Number.
	 */
	public function getDotProduct( p:Point ) : Number
	{
		return (x * p.x) + (y * p.y);
	}
	
	/**
	 * Get cross product.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = p0.getDotProduct( p1 );</em>
	 * @param 	Point.
	 * @return 	Number.
	 */
	public function getCrossProduct( p:Point ) : Number
	{
		return (x * p.y) - (y * p.x);
	}
	
	/**
	 * Multiply each property by a scalar.<br><br>
	 * <strong>Usage :</strong> <em>p0.scalarMultiply( n );</em>
	 * @param 	Number.
	 */
	public function scalarMultiply( n:Number ) : Void
	{
		x *= n;
		y *= n;
	}
	
	/**
	 * Get projection on another Vector.<br><br>
	 * <strong>Usage :</strong> <em>var p:Point = p0.scalarMultiply( p1 );</em>
	 * @param 	Point.
	 * @return 	Point (p0 projected on p1).
	 */

	public function project( p1:Point ) : Point
	{
		var n:Number = p1.getDotProduct(p1);
		if( n == 0)
		{
			//zero-length
			Logger.LOG("**WARNING** " + toString() + ".project() was given a zero-length projection vector!", LogLevel.WARN);
			return clone();
		}
		else
		{
			var p0:Point = p1.clone();
			p0.scalarMultiply( getDotProduct(p1)/n );
			return p0;
		}
	}
	
	public function getProjectionLength( p1:Point ) : Number
	{
		var n:Number = p1.getDotProduct(p1);
		if( n == 0)
		{
			//zero-length
			Logger.LOG("**WARNING** " + toString() + ".getProjectionLength() was given a zero-length projection vector!", LogLevel.WARN);
			return 0;
		}
		else
		{
			var p0:Point = p1.clone();
			return Math.abs(getDotProduct(p1)/n);
		}
	}
	
	/**
	 * 
	 */
	public function toString(): String
	{
		return '[' + String(x) + ', ' + String(y) + ']';
	}
	
	/*
	 * Static methods
	 */
	 
	/**
	 * Minus operation beetween the properties of two Point objects.<br><br>
	 * <strong>Usage :</strong> <em>var v3:Point = Point.minusNew(v1, v2);</em>
	 * @return Point.
	 */
	public static function minusNew(p1:Point, p2:Point) : Point
	{ 
		return new Point(p1.x - p2.x, p1.y - p2.y); 
	}
	
	/**
	 * Plus operation beetween the properties of two Point objects.<br><br>
	 * <strong>Usage :</strong> <em>var p3:Point = Point.plusNew(p1, p2);</em>
	 * @return Point.
	 */
	public static function plusNew(p1:Point, p2:Point) : Point 
	{ 
		return new Point(p1.x + p2.x, p1.y + p2.y); 
	}

	/**
	 * Get a copy of the object with negative properties.<br><br>
	 * <strong>Usage :</strong> <em>var p2:Point = Point.negNew(p1);</em>
	 * @return Point.
	 */
	public static function negNew(p:Point) : Point 
	{ 
		return new Point( -p.x, -p.y); 
	}
	
	/**
	 * Get a copy of the object with absolute properties.<br><br>
	 * <strong>Usage :</strong> <em>var v2:Point = Point.absNew(p1);</em>
	 * @return Point.
	 */
	public static function absNew(p:Point) : Point 
	{ 
		return new Point( Math.abs(p.x), Math.abs(p.y) ); 
	}
	
	/**
	 * Get distance between two points.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = Point.getDistance();</em>
	 * @return Number.
	 */
	public static function getDistance(p1:Point, p2:Point) : Number
	{
		return Point.minusNew(p1, p2).getLength();
	}
}