/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.EventBroadcaster;
import com.bourre.transitions.BasicTweenFPS;
import com.bourre.transitions.ITween;
import com.bourre.transitions.ITweenListener;
import com.bourre.transitions.TweenEvent;
import com.bourre.transitions.TweenEventType;

class com.bourre.transitions.TweenFPS extends BasicTweenFPS 
	implements ITween
{
	private var _oEB:EventBroadcaster;
	
	/**
	@summary Broadcasted to listeners when tween starts.<br>
	@see com.bourre.transitions.TweenEvent
	 */
	[Event("onStart")]
	public static var onStartEVENT:TweenEventType = TweenEventType.onStartEVENT;
	
	/**
	@summary Broadcasted to listeners when tween stops.<br>
	@see com.bourre.transitions.TweenEvent
	 */
	[Event("onStop")]
	public static var onStopEVENT:TweenEventType = TweenEventType.onStopEVENT;
	
	/**
	@summary Broadcasted to listeners when tween is finished.<br>
	@see com.bourre.transitions.TweenEvent
	 */
	[Event("onMotionFinished")]
	public static var onMotionFinishedEVENT:TweenEventType = TweenEventType.onMotionFinishedEVENT;
	
	/**
	@summary Broadcasted to listeners when property value is updated.<br>
	@see com.bourre.transitions.TweenEvent
	 */
	[Event("onMotionChanged")]
	public static var onMotionChangedEVENT:TweenEventType = TweenEventType.onMotionChangedEVENT;

	private var _eOnMotionChangedEvent : TweenEvent;
	private var _eOnMotionFinishedEvent : TweenEvent;
	private var _eOnMotionStartEvent : TweenEvent;
	private var _eOnMotionStopEvent : TweenEvent;
	
	/**
	 * Makes tween on object property value and allows listeners.
	 * <strong>Usage :</strong> <em>var t:TweenFPS = new TweenFPS(mc, '_alpha', 0, 30, 100);</em>
	 * @param oT 	Object which the Tween targets.
	 * @param sP 	Setter (method or property).
	 * @param nE 	Ending value of property.
	 * @param nFps 	Length of time of the motion in frames.
	 * @param nS 	Starting value of property (optional).
	 * @param fE 	Easing function (optional).
	*/
	public function TweenFPS(oT, sP, nE:Number, nMs:Number, nS:Number, fE:Function)
	{
		super(oT, sP, nE, nMs, nS, fE);
		_oEB = new EventBroadcaster( this );
		
		_eOnMotionStartEvent = new TweenEvent(onStartEVENT, this);
		_eOnMotionStopEvent = new TweenEvent(onStopEVENT, this);
		_eOnMotionChangedEvent = new TweenEvent(onMotionChangedEVENT, this);
		_eOnMotionFinishedEvent = new TweenEvent(onMotionFinishedEVENT, this);
	}

	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements ITweenListener interface.
	 */
	public function addListener(oL:ITweenListener) : Void
	{
		_oEB.addListener(oL);
	}
	
	/**
	 * Remove listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.removeListener(myListener);</em>
	 * @param oL Listener object which implements ITweenListener interface.
	 */
	public function removeListener(oL:ITweenListener) : Void
	{
		_oEB.removeListener(oL);
	}
	
	/**
	 * Add listener for specifical event.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(TweenFPS.onMotionFinishedEVENT, myListener);</em>
	 * @param t Name of the Event.
	 * @param oL Listener object.
	 */
	public function addEventListener(e:TweenEventType, oL) : Void
	{
		_oEB.addEventListener(e, oL);
	}
	
	/**
	 * Remove listener for specifical event.<br><br>
	 * <strong>Usage :</strong> <em>t.removeListener(TweenFPS.onMotionFinishedEVENT, myListener);</em>
	 * @param t Name of the Event.
	 * @param oL Listener object.
	 */
	public function removeEventListener(e:TweenEventType, oL) : Void
	{
		_oEB.removeEventListener(e, oL);
	}
	
	/**
	 * Call start function.<br><br>
	 * <strong>Usage :</strong> <em>t.start();</em>
	 */
	public function start() : Void
	{
		execute();
	}
	
	/**
	 * Stop tweening.<br><br>
	 * <strong>Usage :</strong> <em>t.stop();</em>
	 */
	public function stop() : Void
	{
		super.stop();
		_oEB.broadcastEvent( _eOnMotionStopEvent );
	}
	
	/**
	 * Start tweening. (Command polymorphism).<br><br>
	 * <strong>Usage :</strong> <em>t.execute();</em>
	 * @see com.bourre.commands.Command
	 */
	public function execute() : Void
	{
		super.execute();
		_oEB.broadcastEvent( _eOnMotionStartEvent );
	}
		
	public function toString() : String
	{
		return 'TweenFPS' + HashCodeFactory.getKey( this );
	}
	
	/**
	 *	Implemented to add tween as listener to another tween.
	 */
	public function onMotionFinished() : Void
	{
		execute();
	}

	
	//
	private function _onMotionEnd() : Void
	{
		super._onMotionEnd();
		_oEB.broadcastEvent( _eOnMotionChangedEvent );
		_oEB.broadcastEvent( _eOnMotionFinishedEvent );
	}
	
	private function _onUpdate() : Void
	{
		super._onUpdate();
		_oEB.broadcastEvent( _eOnMotionChangedEvent );
	}
}